@@ -78,8 +78,8 @@ int exist_file(const char *filename);
 int is_regular_file(const char *filename);
 astr agetcwd(void);
 astr get_home_dir(void);
-int expand_path(const char *path, astr dir, astr fname);
-astr compact_path(const char *path);
+astr expand_path(astr path);
+astr compact_path(const astr path);
 void read_from_disk(const char *filename);
 int find_file(const char *filename);
 Completion *make_buffer_completion(void);
