@@ -20,8 +20,6 @@
    Software Foundation, Fifth Floor, 51 Franklin Street, Boston, MA
    02111-1301, USA.  */
 
-/*	$Id: main.c,v 1.108 2007/06/05 22:30:22 rrt Exp $	*/
-
 #include "config.h"
 
 #include <assert.h>
@@ -217,28 +215,6 @@ struct option longopts[] = {
     { 0, 0, 0, 0 }
 };
 
-static astr expand_file(char *path)
-{
-  int ret;
-  astr dir = astr_new();
-  astr fname = astr_new();
-  astr buf = astr_new();
-
-  if ((ret = expand_path(path, dir, fname))) {
-    astr_cpy_cstr(buf, astr_cstr(dir));
-    astr_cat_cstr(buf, astr_cstr(fname));
-  }
-
-  astr_delete(dir);
-  astr_delete(fname);
-
-  if (!ret) {
-    astr_delete(buf);
-    return NULL;
-  }
-  return buf;
-}
-
 int main(int argc, char **argv)
 {
   int c, bflag = FALSE, qflag = FALSE;
@@ -347,21 +323,18 @@ int main(int argc, char **argv)
     /* Reinitialise the scratch buffer to catch settings */
     init_buffer(cur_bp);
 
-    if (argc >= 1)
+    if (argc >= 1) {
       while (*argv) {
         size_t line = 1;
         if (**argv == '+')
           line = strtoul(*argv++ + 1, NULL, 10);
         if (*argv) {
-          astr as = expand_file(*argv);
+          astr as = astr_new();
 
-          if (as) {
-            find_file(astr_cstr(as));
-            argv++;
-          } else {
-            fprintf(stderr, "zile: %s: invalid filename or path\n", astr_cstr(*argv));
-            exit(1);
-          }
+          astr_cpy_cstr(as, *argv);
+
+          find_file(astr_cstr(as));
+          argv++;
           astr_delete(as);
 
           if (line > 1)
@@ -369,7 +342,7 @@ int main(int argc, char **argv)
           lastflag |= FLAG_NEED_RESYNC;
         }
       }
-    else if (list_length(fargs) == 0)
+    } else if (list_length(fargs) == 0)
       /* Show the splash screen only if no files and no Lisp expression
          or load file is specified on the command line. */
       about_screen();
